from setuptools import setup

setup(
    name='vtu_aiml',
    version='0.1.2',    
    description='A example Python package',
    url='https://github.com/shuds13/pyexample',
    author='Hrithikesh',
    author_email='hrithikesh@gmail.com',
    license='BSD 2-clause',
    packages=['vtu_aiml'],
    install_requires=[]
)